PRO G_READ_FITS_MAP, filename, T_sky, X_sky, Y_sky, coord

; NAME:
;       G_READ_FITS_MAP

; PURPOSE:
; 	reads a GLESP temperature map in FITS binary table extension
;	in file with 'filename' and return temperature and position.
;	Calculate pixel-coordinates if requested.

; CALLING SEQUENCE:
; 	READ_G_FITS_MAP, filename, T_sky, X_sky, Y_sky [,coord]
 
; INPUTS:
;	filename = String containing the name of the file to be read.

; OUTPUTS:
;	X_sky = FLOAT or DOUBLE 1-dim array with COS(THETA) for each ring
;	Y_sky = LONG integer 1-dim array with number of pixels/ring
;       T_sky = FLOAT or DOUBLE 1-dim array. Dimension = number
;	of pixels		 
;	FLOAT or DOUBLE depends on the format in the input file

; OPTIONAL OUTPUTS:
;	coord  = FLOAT 2-dim array with pixel coordinates in radian.
;		 coord[*,0] is theta (polar angle) in the interval [0,pi]
;			(0 = NORTH-POLE)
;		 coord[*,1] is phi (azimuthal angle in the interval [0,2pi] 

; PROCEDURES USED:
;	requires the THE IDL ASTRONOMY USER'S LIBRARY 
; 	that can be found at http://idlastro.gsfc.nasa.gov/homepage.html

;===========OBS!

;	Program to read GLESP fits files.
;	The program is based on the corresponding HEALPix program
;	which we hereby acknowledge.
;	Per Rex Christensen, September 2004
	
;===========OBS!

forward_function get_map_coord

syntax = 'Syntax : G_READ_FITS_MAP,filename,T_sky,X_sky,Y_sky[,coord]'
syntax2 = '   No File Read, Abort !'
if N_params() LT 4 then begin
    print,syntax
    print,syntax2
    retall
endif

; run astrolib routine to set up non-standard system variables
defsysv, '!DEBUG', EXISTS = i  ; check if astrolib variables have been set-up
if (i ne 1) then astrolib       ; if not, run astrolib to do so

; get the primary header information
if (datatype(filename) ne 'STR') then begin
    print,syntax
    print,'Invalid file name'
    message,syntax2
endif

fullpath = findfile(filename,count=count)
if (count eq 0) then begin
    prim_stc = 0 & xten_stc = 0
    message, 'file '+filename+' not found'
endif

hdr = headfits(filename, errmsg=errmsg)
if (strtrim(errmsg) ne '') then begin
    message,errmsg
endif

; checks for the existence of an extension
; if none, reads the image
fits_info,filename, /silent, n_ext=n_ext
if (n_ext eq 0) then begin
    T_sky = readfits(filename,hdr, silent=silent)
    exthdr = ' '
    return
endif

; identify the format in the row (number of entry per row)
fits_read, filename, junk, exthdr, /header_only, /no_pdu, exten_no=1
tbinfo, exthdr, tab_xhdr
nentry = max(tab_xhdr.numval)
nmaps  = ROUND(FLOAT(SXPAR(exthdr,'TFIELDS')))
naxis2 = LONG(       SXPAR(exthdr,'NAXIS2' ))
ndata = nentry * naxis2
n_wpr = LONG(total(tab_xhdr.width*tab_xhdr.numval)) ; number of bytes per row
type = max(tab_xhdr.idltype) ; find out type of output array to fit all data
junk = where(tab_xhdr.idltype eq 7, n_string)

if (n_string ge 1) then begin
    message,'Table in input file contains strings'
endif

; find bytes to read
max_byte = naxis2*n_wpr - 1L
max_pix  = ndata - 1L
firstbyte = 0L
lastbyte  = max_byte
firstpix = 0L
lastpix = max_pix

npix = ndata
fits_read, filename, data, exten_no = 1, /no_pdu

if type eq 4 then print,'FITS-FORMAT in: FLOAT'
if type eq 5 then print,'FITS-FORMAT in: DOUBLE'


T_sky = make_array(type=type,tab_xhdr.numval[2],nmaps-2,/nozero)

for i=0,nmaps-1 do begin

    nent = tab_xhdr.numval[i]

    if (i eq 0) then begin
	X_sky	= make_array(type =tab_xhdr.idltype[i],nent,/nozero)
	X_sky	= tbget(tab_xhdr, data, i+1)
	goto,next
     endif 

     if (i eq 1) then begin
	Y_sky	= make_array(type =tab_xhdr.idltype[i],nent,/nozero)
	Y_sky	= tbget(tab_xhdr, data, i+1)
	goto,next
     endif

     T_sky[*,i-2] = tbget(tab_xhdr, data, i+1)	

     next:

endfor
data = 0

help,T_sky

if n_params() gt 4 then coord	= get_map_coord(x_sky,y_sky)

fin:
return
end

;---------------------------------------------------------------------------------

function get_map_coord, x_sky, y_sky 

n	= long(total(y_sky,/double) +0.5 )
print,'NUMBER of PIXELS: ', n

coord	= fltarr(n,2)

n_x     = n_elements(x_sky)

k	= -1L
for j = 0, n_x -1L do begin
	theta	= acos(x_sky[j])
	delta_phi	= 2.0D*!DPI/y_sky[j]
	for i = 0, y_sky[j] -1 do begin
		k	= k +1L
		coord[k,0]	= theta
		coord[k,1]	= i*delta_phi
	endfor
endfor

return, coord
end